@@ -4,6 +4,12 @@
 
 #define FF_CPU_TEMP_UNSET (0/0.0)
 
+typedef struct FFCPUCore
+{
+    uint32_t freq;
+    uint32_t count;
+} FFCPUCore;
+
 typedef struct FFCPUResult
 {
     FFstrbuf name;
@@ -13,12 +19,14 @@ typedef struct FFCPUResult
     uint16_t coresLogical;
     uint16_t coresOnline;
 
-    double frequencyBase; // GHz
-    double frequencyMax; // GHz
-    double frequencyMin; // GHz
+    uint32_t frequencyBase; // GHz
+    uint32_t frequencyMax; // GHz
+    uint32_t frequencyBiosLimit; // GHz
+
+    FFCPUCore coreTypes[16]; // number of P cores, E cores, etc.
 
     double temperature;
 } FFCPUResult;
 
-const char* ffCPUDetectByCpuid(FFCPUResult* cpu);
 const char* ffDetectCPU(const FFCPUOptions* options, FFCPUResult* cpu);
+const char* ffCPUAppleCodeToName(uint32_t code);
