@@ -46,6 +46,8 @@ static void detectQualcomm(FFCPUResult* cpu)
     else if (ffStrbufEqualS(&cpu->name, "SM6450"))
         ffStrbufSetStatic(&cpu->name, "Qualcomm Snapdragon 6 Gen 1 [SM6450]");
 
+    else if (ffStrbufEqualS(&cpu->name, "SM4635"))
+        ffStrbufSetStatic(&cpu->name, "Qualcomm Snapdragon 4s Gen 2 [SM4635]");
     else if (ffStrbufEqualS(&cpu->name, "SM4450"))
         ffStrbufSetStatic(&cpu->name, "Qualcomm Snapdragon 4 Gen 2 [SM4450]");
     else if (ffStrbufEqualS(&cpu->name, "SM4375"))
@@ -72,19 +74,74 @@ static void detectAndroid(FFCPUResult* cpu)
 }
 #endif
 
-static const char* parseCpuInfo(FFCPUResult* cpu, FFstrbuf* physicalCoresBuffer, FFstrbuf* cpuMHz, FFstrbuf* cpuIsa, FFstrbuf* cpuUarch)
-{
-    FF_AUTO_CLOSE_FILE FILE* cpuinfo = fopen("/proc/cpuinfo", "r");
-    if(cpuinfo == NULL)
-        return "fopen(\"/proc/cpuinfo\", \"r\") failed";
+#if __arm__ || __aarch64__
+#include "cpu_arm.h"
 
+static void detectArmName(FILE* cpuinfo, FFCPUResult* cpu, uint32_t implId)
+{
+    FF_AUTO_FREE char* line = NULL;
+    rewind(cpuinfo);
+    size_t len = 0;
+    uint32_t lastPartId = UINT32_MAX;
+    uint32_t num = 0;
+    while(getline(&line, &len, cpuinfo) != -1)
+    {
+        if (!ffStrStartsWith(line, "CPU part\t: ")) continue;
+        uint32_t partId = (uint32_t) strtoul(line + strlen("CPU part\t: "), NULL, 16);
+        const char* name = NULL;
+        if (partId > 0) // Linux reports 0 for unknown CPUs
+        {
+            switch (implId)
+            {
+                case 0x41: name = armPartId2name(partId); break;
+                case 0x42: name = brcmPartId2name(partId); break;
+                case 0x43: name = caviumPartId2name(partId); break;
+                case 0x44: name = decPartId2name(partId); break;
+                case 0x46: name = fujitsuPartId2name(partId); break;
+                case 0x48: name = hisiPartId2name(partId); break;
+                case 0x4e: name = nvidiaPartId2name(partId); break;
+                case 0x50: name = apmPartId2name(partId); break;
+                case 0x51: name = qcomPartId2name(partId); break;
+                case 0x53: name = samsungPartId2name(partId); break;
+                case 0x56: name = marvellPartId2name(partId); break;
+                case 0x61: name = applePartId2name(partId); break;
+                case 0x66: name = faradayPartId2name(partId); break;
+                case 0x69: name = intelPartId2name(partId); break;
+                case 0x6d: name = msPartId2name(partId); break;
+                case 0x70: name = ftPartId2name(partId); break;
+                case 0xc0: name = amperePartId2name(partId); break;
+            }
+        }
+        if (lastPartId != partId)
+        {
+            if (lastPartId != UINT32_MAX)
+            {
+                if (num > 1)
+                    ffStrbufAppendF(&cpu->name, "*%u", num);
+                ffStrbufAppendS(&cpu->name, " + ");
+            }
+            if (name)
+                ffStrbufAppendS(&cpu->name, name);
+            else if (partId)
+                ffStrbufAppendF(&cpu->name, "%s-%X", cpu->vendor.chars, partId);
+            else
+                ffStrbufAppend(&cpu->name, &cpu->vendor);
+            lastPartId = partId;
+            num = 1;
+        }
+        else
+            ++num;
+    }
+    if (num > 1)
+        ffStrbufAppendF(&cpu->name, "*%u", num);
+}
+#endif
+
+static const char* parseCpuInfo(FILE* cpuinfo, FFCPUResult* cpu, FFstrbuf* physicalCoresBuffer, FFstrbuf* cpuMHz, FFstrbuf* cpuIsa, FFstrbuf* cpuUarch, FFstrbuf* cpuImplementer)
+{
     FF_AUTO_FREE char* line = NULL;
     size_t len = 0;
 
-    #ifdef __aarch64__
-    FF_STRBUF_AUTO_DESTROY implementer = ffStrbufCreate();
-    #endif
-
     while(getline(&line, &len, cpuinfo) != -1)
     {
         //Stop after the first CPU
@@ -99,35 +156,22 @@ static const char* parseCpuInfo(FFCPUResult* cpu, FFstrbuf* physicalCoresBuffer,
             ffParsePropLine(line, "isa :", cpuIsa) ||
             ffParsePropLine(line, "uarch :", cpuUarch) ||
 
-            #ifdef __aarch64__
-            (cpu->vendor.length == 0 && ffParsePropLine(line, "CPU implementer :", &implementer)) ||
+            #if __arm__ || __aarch64__
+            (cpu->vendor.length == 0 && ffParsePropLine(line, "CPU implementer :", cpuImplementer)) ||
             #endif
-
+            #if __ANDROID__
             (cpu->name.length == 0 && ffParsePropLine(line, "Hardware :", &cpu->name)) || //For Android devices
+            #endif
+            #if __powerpc__ || __powerpc
             (cpu->name.length == 0 && ffParsePropLine(line, "cpu     :", &cpu->name)) || //For POWER
-            (cpu->name.length == 0 && ffParsePropLine(line, "cpu model               :", &cpu->name)) //For MIPS
+            #endif
+            #if __mips__
+            (cpu->name.length == 0 && ffParsePropLine(line, "cpu model               :", &cpu->name)) || //For MIPS
+            #endif
+            false
         );
     }
 
-    #ifdef __aarch64__
-    // https://github.com/util-linux/util-linux/blob/2cd89de14549d2b2c079a4f8b73f75500d229fee/sys-utils/lscpu-arm.c#L286
-    if (cpu->vendor.length == 0 && implementer.length > 2 /* 0xX */)
-    {
-        uint32_t implId = (uint32_t) strtoul(implementer.chars, NULL, 16);
-        switch (implId)
-        {
-        case 0x41: ffStrbufSetStatic(&cpu->vendor, "ARM"); break;
-        case 0x42: ffStrbufSetStatic(&cpu->vendor, "Broadcom"); break;
-        case 0x48: ffStrbufSetStatic(&cpu->vendor, "HiSilicon"); break;
-        case 0x4e: ffStrbufSetStatic(&cpu->vendor, "Nvidia"); break;
-        case 0x51: ffStrbufSetStatic(&cpu->vendor, "Qualcomm"); break;
-        case 0x53: ffStrbufSetStatic(&cpu->vendor, "Samsung"); break;
-        case 0x61: ffStrbufSetStatic(&cpu->vendor, "Apple"); break;
-        case 0x69: ffStrbufSetStatic(&cpu->vendor, "Intel"); break;
-        }
-    }
-    #endif
-
     return NULL;
 }
 
@@ -277,14 +321,19 @@ void detectAsahi(FFCPUResult* cpu)
 
 const char* ffDetectCPUImpl(const FFCPUOptions* options, FFCPUResult* cpu)
 {
+    FF_AUTO_CLOSE_FILE FILE* cpuinfo = fopen("/proc/cpuinfo", "r");
+    if(cpuinfo == NULL)
+        return "fopen(\"/proc/cpuinfo\", \"r\") failed";
+
     cpu->temperature = options->temp ? detectCPUTemp() : FF_CPU_TEMP_UNSET;
 
     FF_STRBUF_AUTO_DESTROY physicalCoresBuffer = ffStrbufCreate();
     FF_STRBUF_AUTO_DESTROY cpuMHz = ffStrbufCreate();
     FF_STRBUF_AUTO_DESTROY cpuIsa = ffStrbufCreate();
     FF_STRBUF_AUTO_DESTROY cpuUarch = ffStrbufCreate();
+    FF_STRBUF_AUTO_DESTROY cpuImplementerStr = ffStrbufCreate();
 
-    const char* error = parseCpuInfo(cpu, &physicalCoresBuffer, &cpuMHz, &cpuIsa, &cpuUarch);
+    const char* error = parseCpuInfo(cpuinfo, cpu, &physicalCoresBuffer, &cpuMHz, &cpuIsa, &cpuUarch, &cpuImplementerStr);
     if (error) return error;
 
     cpu->coresLogical = (uint16_t) get_nprocs_conf();
@@ -309,83 +358,19 @@ const char* ffDetectCPUImpl(const FFCPUOptions* options, FFCPUResult* cpu)
         ffStrbufAppend(&cpu->name, &cpuIsa);
     }
 
-    #ifdef __ANDROID__
-    detectAndroid(cpu);
-    #endif
+    #if __arm__ || __aarch64__
+    uint32_t cpuImplementer = (uint32_t) strtoul(cpuImplementerStr.chars, NULL, 16);
+    ffStrbufSetStatic(&cpu->vendor, hwImplId2Vendor(cpuImplementer));
 
-    #if defined(__linux__) && defined(__aarch64__)
+    #if __ANDROID__
+    detectAndroid(cpu);
+    #elif __aarch64__
     detectAsahi(cpu);
     #endif
 
     if (cpu->name.length == 0)
-    {
-        FF_STRBUF_AUTO_DESTROY buffer = ffStrbufCreate();
-        if (ffProcessAppendStdOut(&buffer, (char *const[]) { "lscpu", NULL }) == NULL)
-        {
-            char* pstart = buffer.chars;
-
-            if (cpu->vendor.length == 0)
-            {
-                pstart = strstr(pstart, "Vendor ID:");
-                if (pstart)
-                {
-                    pstart += strlen("Vendor ID:");
-                    while (isspace(*pstart)) ++pstart;
-                    if (*pstart)
-                    {
-                        char* pend = strchr(pstart, '\n');
-                        if (pend != NULL)
-                            ffStrbufAppendNS(&cpu->vendor, (uint32_t) (pend - pstart), pstart);
-                        else
-                        {
-                            ffStrbufAppendS(&cpu->vendor, pstart);
-                        }
-                        pstart = pend + 1;
-                        if (pstart >= buffer.chars + buffer.length)
-                            return NULL;
-                    }
-                }
-                else
-                {
-                    pstart = buffer.chars;
-                }
-            }
-
-            while ((pstart = strstr(pstart, "Model name:")))
-            {
-                pstart += strlen("Model name:");
-                while (isspace(*pstart)) ++pstart;
-                if (*pstart == '\0')
-                    break;
-
-                if (cpu->name.length > 0)
-                    ffStrbufAppendS(&cpu->name, " + ");
-
-                if (*pstart == '-')
-                {
-                    if (cpu->vendor.length > 0)
-                        ffStrbufAppend(&cpu->name, &cpu->vendor);
-                    else
-                        ffStrbufAppendS(&cpu->name, "Unknown");
-                    ++pstart;
-                    continue;
-                }
-
-                char* pend = strchr(pstart, '\n');
-                if (pend != NULL)
-                    ffStrbufAppendNS(&cpu->name, (uint32_t) (pend - pstart), pstart);
-                else
-                {
-                    ffStrbufAppendS(&cpu->name, pstart);
-                    break;
-                }
-
-                pstart = pend + 1;
-                if (pstart >= buffer.chars + buffer.length)
-                    return NULL;
-            }
-        }
-    }
+        detectArmName(cpuinfo, cpu, cpuImplementer);
+    #endif
 
     return NULL;
 }
