@@ -20,7 +20,7 @@ void ffPrintWMTheme(FFWMThemeOptions* options)
         else
         {
             FF_PRINT_FORMAT_CHECKED(FF_WMTHEME_DISPLAY_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, FF_WMTHEME_NUM_FORMAT_ARGS, ((FFformatarg[]){
-                {FF_FORMAT_ARG_TYPE_STRBUF, &themeOrError}
+                {FF_FORMAT_ARG_TYPE_STRBUF, &themeOrError, "result"},
             }));
         }
     }
@@ -80,7 +80,7 @@ void ffGenerateWMThemeJsonResult(FF_MAYBE_UNUSED FFWMThemeOptions* options, yyjs
 void ffPrintWMthemeHelpFormat(void)
 {
     FF_PRINT_MODULE_FORMAT_HELP_CHECKED(FF_WMTHEME_MODULE_NAME, "{1}", FF_WMTHEME_NUM_FORMAT_ARGS, ((const char* []) {
-        "WM theme"
+        "WM theme - result",
     }));
 }
 
@@ -97,7 +97,7 @@ void ffInitWMThemeOptions(FFWMThemeOptions* options)
         ffPrintWMthemeHelpFormat,
         ffGenerateWMThemeJsonConfig
     );
-    ffOptionInitModuleArg(&options->moduleArgs);
+    ffOptionInitModuleArg(&options->moduleArgs, "󰓸");
 }
 
 void ffDestroyWMThemeOptions(FFWMThemeOptions* options)
